/*
 * VectorStream 1.4
 * Copyright (c) 2002, Ichiroh Kanaya
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * $Id: vec.c,v 1.3 2003/07/11 06:41:33 kanaya Exp $
 *
 */

#include <ctype.h>
#include <memory.h>
#include <stdlib.h>
#include "vec.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifndef PACKAGE
#define PACKAGE "Unknown (vectorstream)"
#endif

#ifndef VERSION
#define VERSION "Unknwon (1.4 or higher)"
#endif

static int skip_whitespace(FILE *fin) {
  int c;

  do {
    if (feof(fin))
      return 1;
    c = fgetc(fin);
  }
  while (isspace(c));
  ungetc(c, fin);
  return 0;
}

static int skip_comment(FILE *fin) {
  int c;

  skip_whitespace(fin);
  c = fgetc(fin);
  if (c == '%') {
    /* skip to EOL */
    do {
      c = fgetc(fin);
    }
    while (c != '\n');
    /* check the next line (recursive call) */
    return skip_comment(fin);
  }
  else {
    ungetc(c, fin);
    return skip_whitespace(fin);
  }
}

static char *get_token(FILE *fin) {
  /* Warning: this function returns MUTABLE array. */
  static int buff_length = 64;
  static char *buff = 0;
  int i = 0;
  int c;

  if (!buff) {
    buff = (char *)calloc(buff_length, sizeof(char));
  }
  skip_whitespace(fin);
  while (!feof(fin) && !isspace(c = fgetc(fin))) {
    if (i > buff_length - 2) {
      char *tmp_buff;

      tmp_buff = (char *)calloc(buff_length * 2, sizeof(char));
      memcpy(tmp_buff, buff, buff_length * sizeof(char));
      free(buff);
      buff_length *= 2;
      buff = tmp_buff;
    }
    buff[i] = c;
    ++i;
  }
  buff[i] = 0;
  return &buff[0];
}

void fputheader(FILE *fout) {
  fputs("%!VCTR\n%-format=1.1\n%-type=float\n"
	"%%  Created by VectorStream (" VERSION ")\n",
	fout);
}

void fputm(const char *comment, FILE *fout) {
  fputs("%%  ", fout);
  fputs(comment, fout);
  fputc('\n', fout);
}

void fputfv(int n, const float *p, int r, FILE *fout) {
  fprintf(fout, "%d %% Number of elements\n", n);
  if (n > 0) {
    if (r > 1) {
      int i, j;
      for (i = 0; i < n; i += r) {
	for (j = 0; j < r; ++j) {
	  fprintf(fout, "%g ", p[i + j]);
	}
	fputc('\n', fout);
	/* fprintf(fout, "%% This line starts with element num. %u\n", i); */
      }
    }
    else {
      int i;
      for (i = 0; i < n; ++i) {
	fprintf(fout, "%g\n", p[i]);
      }
    }
    fputc('\n', fout);
  }
}

void fgetfv(int *n, float **p, FILE *fin) {
  int i;

  skip_comment(fin);
  *n = atoi(get_token(fin));
  if (*n > 0) {
    *p = (float *)calloc(*n, sizeof(float));
    for (i = 0; i < *n; ++i) {
      skip_comment(fin);
      (*p)[i] = atof(get_token(fin));
    }
  }
  else {
    *p = NULL;
  }
}

void fputiv(int n, const int *p, int r, FILE *fout) {
  fprintf(fout, "%d %% Number of elements\n", n);
  if (n > 0) {
    if (r > 1) {
      int i, j;
      for (i = 0; i < n; i += r) {
	for (j = 0; j < r; ++j) {
	  fprintf(fout, "%d ", p[i + j]);
	}
	fputc('\n', fout);
	/* fprintf(fout, "%% This line starts with element num. %u\n", i); */
      }
    }
    else {
      int i;
      for (i = 0; i < n; ++i) {
	fprintf(fout, "%d\n", p[i]);
      }
    }
    fputc('\n', fout);
  }
}

void fgetiv(int *n, int **p, FILE *fin) {
  int i;

  skip_comment(fin);
  *n = atoi(get_token(fin));
  if (*n > 0) {
    *p = (int *)calloc(*n, sizeof(int));
    for (i = 0; i < *n; ++i) {
      skip_comment(fin);
      (*p)[i] = atoi(get_token(fin));
    }
  }
  else {
    *p = NULL;
  }
}

void fputhv(int n, const short *p, int r, FILE *fout) {
  fprintf(fout, "%d %% Number of elements\n", n);
  if (n > 0) {
    if (r > 1) {
      int i, j;
      for (i = 0; i < n; i += r) {
	for (j = 0; j < r; ++j) {
	  fprintf(fout, "%d ", (int)(p[i + j]));
	}
	fputc('\n', fout);
	/* fprintf(fout, "%% This line starts with element num. %u\n", i); */
      }
    }
    else {
      int i;
      for (i = 0; i < n; ++i) {
	fprintf(fout, "%d\n", (int)(p[i]));
      }
    }
    fputc('\n', fout);
  }
}

void fgethv(int *n, short **p, FILE *fin) {
  int i;

  skip_comment(fin);
  *n = atoi(get_token(fin));
  if (*n > 0) {
    *p = (short *)calloc(*n, sizeof(short));
    for (i = 0; i < *n; ++i) {
      skip_comment(fin);
      (*p)[i] = (short)atoi(get_token(fin));
    }
  }
  else {
    *p = NULL;
  }
}
